#include <sdcc-lib.h>
#include <pic12f675.h>	// /usr/share/sdcc/include/pic/pic12f675.h
#include "inline.h"

//__code unsigned char at 0x2000 id_location[] = {'A', 'B', 'C', 'D', 0, 0, 0 };

unsigned int at 0x2007 __config =
	_CPD_OFF &
	_CP_OFF &
	_BODEN_ON &
	_MCLRE_OFF &
	_PWRTE_OFF &
	_WDT_OFF &
	_INTRC_OSC_NOCLKOUT;

__code unsigned char at 0x2100 eedata[] = {'d', 'u', 'm', 'm', 'y', 0 };

#define	LED1	GPIO0
#define	CARSNS1	GPIO1
#define	CARSNS2	GPIO2
#define RX_BIT		3
#define TX_BIT		4

volatile unsigned char waitCntS;
volatile unsigned char waitCntM;
volatile unsigned char waitCntL;

void waitS();
void waitM();
void waitL();

#define waitSS(x) LD_A(x); waitS();
#define waitMM(x) LD_A(x); waitM();
#define waitLL(x) LD_A(x); waitL();

void transmitSerial(unsigned char);

volatile unsigned char serial_count;
volatile unsigned char serial_work;

void Intr(void) __interrupt 0 {  
	T0IF = 0;	// Clear timer interrupt
}

/*			  _______
	Vcc		-|		 |-		GND
	NC		-|5		0|-		LED
	TX		-|4		1|-		CARSNS1
	NC		-|3_____2|-		CARSNS2
*/

void main(void) {

	unsigned char last_state = 0;
	unsigned char now_state;
	unsigned char change_state;
	unsigned char led_lag = 1;

			//    543210
	GPIO	= 0b00000000;	// GPIO3: Input Only
	CMCON	= 0b00000111;
	ANSEL	= 0b00000000;
	TRISIO	= 0b00001110;	// 1: 入力, 0: 出力

	// initial wait
	LED1 = 1;
	waitLL(10);

	while(1) {
		if(led_lag > 0 && (led_lag -= 1) == 0)
			LED1 = 0;

		now_state = GPIO & 0b00000110;
		if(change_state = now_state ^ last_state) {
			if(change_state & 0b00000010) {
				if(now_state & 0b00000010) {
					transmitSerial(0x01);
					LED1 = 1;
					led_lag = 250;
				} else {
					transmitSerial(0x11);
				}
			}
			if(change_state & 0b00000100) {
				if(now_state & 0b00000100) {
					transmitSerial(0x02);
					LED1 = 1;
					led_lag = 250;
				} else {
					transmitSerial(0x12);
				}
			}
			last_state = now_state;
		}
	}
}

/*------------------------------------------------------------------------------
 *
 *  シリアル送信処理
 *
 *		2400bps: 416.6us, 9600bps: 104.2us, 
 *
 */
void transmitSerial(unsigned char c) {
						serial_work = c;
	_asm
		bcf	GPIO, TX_BIT		;							RES		TX_BIT, (GPIO)					; 104   スタートビット設定(LOW)
		nop				;							NOP										; 001
		nop				;							NOP										; 002
		movlw	8			;							LD		A, 8							; 003
		movwf	_serial_count		;							LD		(_serial_count), A				; 004
transSer1:	movlw	30			;			transSer1:		LD		A, 30							;       30 x 3 + 4 = 94us
		call	WAITnx3p5u		;							CALL	WAITnx3p5u						; 098                   の時間待ち
		nop				;							NOP										; 099
		nop				;							NOP										; 100
		rrf	_serial_work, F		;							RR		(_serial_work)					; 101   [rs232cWork] >> [C]
		btfss	STATUS, 0		;							SKIP	C								; 102
		bcf	GPIO, TX_BIT		;							RES		TX_BIT, (GPIO)					; 103
		btfsc	STATUS, 0		;							SKIP	NC								; 104   データビット設定(HIGH/LOW)
		bsf	GPIO, TX_BIT		;							SET		TX_BIT, (GPIO)					; 001
		decfsz	_serial_count, F	
		goto	transSer1		;							DJNZ	(_serial_count), transSer1		; 002,003,(004)
						;	
		movlw	32			;							LD		A, 32							;       32 x 3 + 4 = 100us
		call	WAITnx3p5u		;							CALL	WAITnx3p5u						; 103                   の時間待ち
		bsf	GPIO, TX_BIT		;							SET		TX_BIT, (GPIO)					; 104   ストップビット設定(HIGH)
						;	
		movlw	32			;							LD		A, 32							;       32 x 3 + 4 = 100us
		call	WAITnx3p5u		;							CALL	WAITnx3p5u						; 100                   の時間待ち
		return				;							RET										; 101,102
	_endasm;
}

void waitS() {
	_asm
						;			;-------------------------------------------------------------------------------
						;			;
						;			;	時間待ち(小)
						;			;
						;			;	    * 4MHz 時
						;			;				LD		A, 10					; 10を指定すると、
						;			;				CALL	WAITnx3p5u				; 2行で 10 x 3 + 5 = 35us 消費
						;			;
WAITnx3p5u:					;			WAITnx3p5u:
		movwf	_waitCntS		;							LD		(_waitCntS), A
WAITnx3p5u1:					;			WAITnx3p5u1:
		decfsz	_waitCntS, F		
		goto	WAITnx3p5u1		;							DJNZ	(_waitCntS), WAITnx3p5u1
						;			;				RET
	_endasm;
}

void waitM() {
	_asm
						;			;-------------------------------------------------------------------------------
						;			;
						;			;	時間待ち(中)
						;			;
						;			;	    * 4MHz 時
						;			;				LD		A, 20					; 0.01秒の場合、20を指定すると、
						;			;				CALL	WAITnx500m35u			; 2行で 20 x 500 - 35 = 9.965ms 消費する
						;			;
WAITnx500m35u:					;			WAITnx500m35u:
		movwf	_waitCntM		;							LD		(_waitCntM), A
		goto	WAITnx500m35u2		;							JP		WAITnx500m35u2
WAITnx500m35u1:	nop				;			WAITnx500m35u1:	NOP
		movlw	12			;							LD		A, 12					;  12 x 3 + 5 =  41us
		call	WAITnx3p5u		;							CALL	WAITnx3p5u				;		の時間待ち
WAITnx500m35u2:	movlw	150			;			WAITnx500m35u2:	LD		A, 150					; 150 x 3 + 5 = 455us
		call	WAITnx3p5u		;							CALL	WAITnx3p5u				;		の時間待ち
		decfsz	_waitCntM, F		
		goto	WAITnx500m35u1		;							DJNZ	(_waitCntM), WAITnx500m35u1
						;			;				RET
	_endasm;
}

void waitL() {
	_asm
						;			;-------------------------------------------------------------------------------
						;			;
						;			;	時間待ち(大)
						;			;
						;			;	    * 4MHz 時
						;			;				LD		A, 10					; 1秒の場合、10を指定すると、
						;			;				CALL	WAITnx01sm25u			; 2行で 10 x 100ms - 25us 消費する
						;			;
WAITnx01sm25u:					;			WAITnx01sm25u:
		movwf	_waitCntL		;							LD		(_waitCntL), A
		goto	WAITnx01sm25u2		;							JP		WAITnx01sm25u2
WAITnx01sm25u1:	movlw	9			;			WAITnx01sm25u1:	LD		A, 9
		call	WAITnx3p5u		;							CALL	WAITnx3p5u
WAITnx01sm25u2:	movlw	200			;			WAITnx01sm25u2:	LD		A, 200
		call	WAITnx500m35u		;							CALL	WAITnx500m35u
		decfsz	_waitCntL, F		
		goto	WAITnx01sm25u1		;							DJNZ	(_waitCntL), WAITnx01sm25u1
						;			;				RET
	_endasm;
}

