#!/usr/bin/env ruby

require 'tk'

#---------------------------------------------------------------
#
#	ラップセグメントクラス
#
class LapSegs

	@@styles = {
		:CAP => {
			:FONT => 'helvetica 10 bold',
			:FG => 'white',
			:BG => 'black',
		},
		:DIGI => {
			:FONT => 'Pocket\ Calculator 40',
			:FG => 'orange',
			:BG => 'black',
		},
	}

	@@letters = [
		{	:STYLE => :CAP,		:TEXT => "dummy",	},
		{	:STYLE => :DIGI,	:TEXT => "88",		},
		{	:STYLE => :CAP,		:TEXT => "\nlap",	},
		{	:STYLE => :DIGI,	:TEXT => "88",		},
		{	:STYLE => :CAP,		:TEXT => "  ",		},
		{	:STYLE => :DIGI,	:TEXT => "88",		},
		{	:STYLE => :CAP,		:TEXT => "\nsec",	},
		{	:STYLE => :DIGI,	:TEXT => "888",		},
	]

	attr_reader :frame

	def initialize(row)

		@frame = TkFrame.new
		@chunks = []

		n = -1; @@letters.each {|letter|
			wg = TkLabel.new(@frame)
			if(letter[:STYLE] == :DIGI)
				wg.textvariable(ck = TkVariable.new(letter[:TEXT]))
				@chunks << ck
			else
				wg.text(n == -1 ? row[:TEXT] : letter[:TEXT])
			end
			wg.font(@@styles[letter[:STYLE]][:FONT])
			wg.fg(@@styles[letter[:STYLE]][:FG])
			wg.bg(@@styles[letter[:STYLE]][:BG])
			wg.grid('column' => n += 1, 'row' => 0, 'sticky' => 'news')
		}
	end

	def set(info)
		@chunks[0].value = '%02d' % info[0]
		@chunks[1].value = '%02d' % info[1]
		@chunks[2].value = '%02d' % [info[2].to_i % 100]
		@chunks[3].value = '%03d' % [(info[2].to_f * 1000).to_i % 1000]
	end
end

#---------------------------------------------------------------
#
#	ラップボードクラス
#
class LapBoard

	@@rows = [
		{	:ID => :LAST,	:TEXT => 'Last'	},
		{	:ID => :NOW,	:TEXT => 'Now'	},
		{	:ID => :p1,		:TEXT => 'Top'	},
		{	:ID => :p2,		:TEXT => '2nd'	},
		{	:ID => :p3,		:TEXT => '3rd'	},
	]

	def initialize(configs = {
			:MAX_CARS		=> 2,
			:CAR_NOS		=> [27, 86],
			:LAPTIME_FILE	=> 'laptimes.txt',
		})

		@max_cars = configs[:MAX_CARS]
		@car_nos = configs[:CAR_NOS]
		@current_car = 0

		@lines = []; @@rows.each {|row|
			wg = LapSegs.new(row)
			wg.frame.pack('anchor' => 'w')
			@lines << wg
		}

		#-------------------------------------------------------
		#
		#	手動ラップタイム計測ボタン定義
		#
		lp = TkButton.new
		lp.text('=== LAP! ===')
		lp.command(proc { do_lap })
		lp.pack('side' => 'right')

		#-------------------------------------------------------
		#
		#	カーナンバー変更ボタン定義
		#
		cc = TkButton.new
		cc.text('Change Car No.')
		cc.command(proc { change_car_no })
		cc.pack('side' => 'right')

		#-------------------------------------------------------
		#
		#	終了ボタン定義
		#
		ex = TkButton.new
		ex.text('Exit')
		ex.command(proc { save_laptimes; exit })
		ex.pack('side' => 'left')

		#-------------------------------------------------------
		#
		#	計時開始！
		#
		load_laptimes(configs[:LAPTIME_FILE])
		@lap_no = 0; @lastlap = 99.999
		update_laps; @lap_no += 1
		@basetime = Time.now
	end

	#-----------------------------------------------------------
	#
	#	カーナンバーの割り当て変更
	#
	def change_car_no
		@current_car = (@current_car + 1) % @max_cars
	end

	#-----------------------------------------------------------
	#
	#	ラップタイム計測
	#
	def do_lap(car_no = 0)
		now = Time.now
		@laptimes << [@car_nos[(@current_car + car_no) % @max_cars], @lap_no, (@lastlap = now - @basetime)]
		@laptimes.sort!{|a, b| a[2] <=> b[2] }
		update_laps; @lap_no += 1
		@basetime = now
		system('play f1.wav >/dev/null 2>&1 &')	# F1 マシンの通過音を再生！
	end

	#-----------------------------------------------------------
	#
	#	現ラップを更新する
	#
	def update
		@lines[1].set([@car_nos[@current_car], @lap_no, Time.now - @basetime])
	end

	#-----------------------------------------------------------
	#
	#	各ラップタイムを更新する
	#
	def update_laps
		@lines[0].set([@car_nos[@current_car], @lap_no, @lastlap])
		(2...@@rows.size).each {|n|
			@lines[n].set(@laptimes[n - 2])
		}
	end

	#-----------------------------------------------------------
	#
	#	ファイルから記録を読む
	#
	def load_laptimes(filename)
		@laptimes = []; open(@laptimes_file = filename).each {|l|
			ll = l.split(/, /); @laptimes << [ll[0].to_i, 0, ll[1].to_f]
		}
		@laptimes << [99, 0, 99.999] while(@laptimes.size < @@rows.size)
		@laptimes.sort!
	end

	#-----------------------------------------------------------
	#
	#	ファイルへ記録を書き出す
	#
	def save_laptimes(filename = @laptimes_file)
		open(filename, 'w') {|fd|
			max = @@rows.size; @laptimes.each {|l|
				fd.write("%d, %10.6f\n" % [l[0], l[2]])
				(max -= 1) < 1 and break
			}
		}
	end
end

#===============================================================================
#
#	メイン
#
root = TkRoot.new { title('Laptime Board') }

# print current settings - stty -g < /dev/ttyUSB0
$config = Hash.new
$config['tty']  = '/dev/ttyUSB0'
$config['stty'] = '1:0:800008bd:0:3:1c:7f:15:4:5:1:0:11:13:1a:0:12:f:17:16:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0'	#  9600bps
system("stty #{$config['stty']} < #{$config['tty']}")
sensor = open($config['tty'], 'w+')

board = LapBoard.new
root.bind('Key', proc{|k| board.change_car_no })
TkAfter.new(10, -1, proc {	# msec, times
	afds = select([sensor], [], [], 0) and report = afds[0][0].getc
	report == 1 and board.do_lap(0)
	report == 2 and board.do_lap(1)
	board.update
}).start

Tk.mainloop

__END__
